'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var jsxRuntime = require('react/jsx-runtime');

const addToSet = (set, value, is) => {
    if (is) {
        set.add(value);
    }
    else {
        set.delete(value);
    }
    return set;
};
const flattenNodes = (nodes) => {
    const flatMap = new Map();
    const flatten = (node, pid = null, depth = 0) => {
        if (flatMap.has(node.id)) {
            throw Error(`Found duplicate entries in tree for node: ${node.id}`);
        }
        flatMap.set(node.id, {
            ...node,
            pid,
            depth,
        });
        node.children?.forEach((child) => flatten(child, node.id, depth + 1));
    };
    nodes.forEach((node) => flatten(node, null, 0));
    return flatMap;
};
const toggleChildren = (nodeId, isChecked, flatNodes, checkedSet) => {
    const node = flatNodes.get(nodeId);
    if (!node?.children?.length) {
        return;
    }
    node.children.forEach((child) => {
        addToSet(checkedSet, child.id, isChecked);
        if (child.children?.length) {
            toggleChildren(child.id, isChecked, flatNodes, checkedSet);
        }
    });
};
const toggleParent = (nodeId, checkedSet, flatNodes) => {
    const node = flatNodes.get(nodeId);
    if (!node || !node.pid) {
        return;
    }
    const parent = flatNodes.get(node.pid);
    if (!parent || !parent.children || !parent.children.length) {
        return;
    }
    const allChecked = parent.children.every((child) => checkedSet.has(child.id));
    addToSet(checkedSet, parent.id, allChecked);
    toggleParent(parent.id, checkedSet, flatNodes);
};

const useCheckboxTree = (nodes, initialChecked = []) => {
    const [checked, setChecked] = react.useState(initialChecked);
    const flatNodes = react.useMemo(() => flattenNodes(nodes), [nodes]);
    const selectNode = react.useCallback((id, isChecked = true) => {
        const checkedSet = new Set(checked);
        if (!flatNodes.has(id)) {
            return checked;
        }
        addToSet(checkedSet, id, isChecked);
        toggleChildren(id, isChecked, flatNodes, checkedSet);
        toggleParent(id, checkedSet, flatNodes);
        const checkedItems = [...checkedSet];
        setChecked(checkedItems);
        return checkedItems;
    }, [checked, flatNodes]);
    const deSelectNode = react.useCallback((id) => selectNode(id, false), [selectNode]);
    const clear = react.useCallback(() => {
        setChecked([]);
    }, []);
    const state = react.useMemo(() => {
        const nodeState = new Map();
        const isNodeIndeterminate = (node, isChecked) => {
            let isIndeterminate = false;
            if (isChecked) {
                return isIndeterminate;
            }
            const { children = [] } = node;
            // check if child has any checked items
            isIndeterminate = children.some((child) => checked.includes(child.id));
            // if there are no checked items in direct child
            // try deeper by iterating over its child to check if there any checed items inside
            isIndeterminate || (isIndeterminate = children.some((child) => isNodeIndeterminate(child, isChecked)));
            return isIndeterminate;
        };
        flatNodes.forEach((node) => {
            const isChecked = checked.includes(node.id);
            const isIndeterminate = isNodeIndeterminate(node, isChecked);
            const checkboxState = isIndeterminate ? 'indeterminate' : isChecked;
            nodeState.set(node.id, checkboxState);
        });
        return nodeState;
    }, [checked, flatNodes]);
    const indeterminates = react.useMemo(() => {
        return [...state].filter((cbState) => cbState[1] === 'indeterminate').map((cbState) => cbState[0]);
    }, [state]);
    return {
        checked,
        state,
        indeterminates,
        selectNode,
        deSelectNode,
        clear,
    };
};

const CheckboxTreeContext = react.createContext(null);
const useCheckboxTreeContext = () => react.useContext(CheckboxTreeContext);
const CheckboxTreeProvider = ({ children, ...props }) => {
    return jsxRuntime.jsx(CheckboxTreeContext.Provider, { value: props, children: children });
};

exports.CheckboxTreeProvider = CheckboxTreeProvider;
exports.default = useCheckboxTree;
exports.useCheckboxTreeContext = useCheckboxTreeContext;
//# sourceMappingURL=index.cjs.map
